using System;
using System.Collections;
using System.Data;
using System.Configuration;
using System.IO;
using System.Net;
using System.Text;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

namespace ASPItalia.Books.Web.Chapter11
{

    public class FtpDataSourceView : DataSourceView
    {
        private FtpDataSource owner;

        public FtpDataSourceView(FtpDataSource owner, string viewName)
            : base(owner, viewName)
        {
            this.owner = owner;
        }

        protected override IEnumerable ExecuteSelect(DataSourceSelectArguments arguments)
        {
            // Ottengo l'oggetto FtpRequest
            FtpWebRequest request = owner.GetFtpRequest(null);
            request.Method = WebRequestMethods.Ftp.ListDirectoryDetails;

            // Ottengo la risposta con la lista
            using (FtpWebResponse response = (FtpWebResponse)request.GetResponse())
            {
                using (Stream stream = response.GetResponseStream())
                {
                    // Parserizzo il risultato
                    return this.owner.Parser.ParseListDirectoryDetails(stream);
                }
            }
        }

        protected override int ExecuteDelete(IDictionary keys, IDictionary oldValues)
        {
            if (!keys.Contains("Name"))
                throw new HttpException("Cannot found Name key to identify file to delete");
            if (!keys.Contains("IsDirectory"))
                throw new HttpException("Cannot found IsDirectory key to identify if it's a directory or file");
            
            // Ottengo l'oggetto FtpRequest
            FtpWebRequest request = owner.GetFtpRequest(keys["Name"].ToString());
            // In base al campo IsDirectory uso il metodo appropriato
            request.Method = (Convert.ToBoolean(keys["IsDirectory"])) ? WebRequestMethods.Ftp.RemoveDirectory : WebRequestMethods.Ftp.DeleteFile;

            using (FtpWebResponse response = (FtpWebResponse) request.GetResponse())
            {
                return (response.StatusCode == FtpStatusCode.FileActionOK) ? 1 : 0;
            }
        }

        public override bool CanDelete
        {
            get
            {
                return true;
            }
        }
  
    }

}