using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Collections;
using System.Net;
using System.IO;

namespace ASPItalia.Books.Web.Chapter11
{

    public class FtpHierarchicalDataSourceView : HierarchicalDataSourceView
    {

        private FtpDataSource owner;
        private string viewPath;

        internal FtpHierarchicalDataSourceView(FtpDataSource owner, string viewPath) : base()
        {
            this.owner = owner;
            this.viewPath = viewPath;
        }

        public override IHierarchicalEnumerable Select()
        {
            return new FtpHierarchicalEnumerable(this.owner, null, this.viewPath);
        }
    }

    public class FtpHierarchicalEnumerable : IHierarchicalEnumerable
    {

        private FtpDataSource owner;
        private FtpHierarchyData parent;
        private string viewPath;

        internal FtpHierarchicalEnumerable(FtpDataSource owner, FtpHierarchyData parent, string viewPath)
        {
            this.owner = owner;
            this.parent = parent;
            this.viewPath = viewPath;
        }

        #region IHierarchicalEnumerable Members

        IHierarchyData IHierarchicalEnumerable.GetHierarchyData(object enumeratedItem)
        {
            FtpFileInfo file = (FtpFileInfo)enumeratedItem;
            return new FtpHierarchyData(this.owner, parent, file, String.Format("{0}/{1}", this.viewPath, file.Name));
        }

        public class FtpHierarchyData : IHierarchyData
        {
            private FtpFileInfo file;
            private string _viewPath;
            private FtpHierarchyData _parent;
            private FtpDataSource owner;

            internal FtpHierarchyData(FtpDataSource owner, FtpHierarchyData parent, FtpFileInfo file, string viewPath)
            {
                this.owner = owner;
                this._parent = parent;
                this.file = file;
                this._viewPath = viewPath;
            }

            #region IHierarchyData Members

            IHierarchicalEnumerable IHierarchyData.GetChildren()
            {
                if (this.file.IsDirectory)
                    return new FtpHierarchicalEnumerable(this.owner, this, this._viewPath);
                else
                    return null;
            }

            IHierarchyData IHierarchyData.GetParent()
            {
                return _parent;
            }

            bool IHierarchyData.HasChildren
            {
                get { return (this.file.IsDirectory); }
            }

            object IHierarchyData.Item
            {
                get { return this.file; }
            }

            string IHierarchyData.Path
            {
                get { return this._viewPath; }
            }

            string IHierarchyData.Type
            {
                get { return (this.file.IsDirectory) ? "Directory" : "File"; }
            }

            #endregion
        }

        #endregion

        #region IEnumerable Members

        IEnumerator IEnumerable.GetEnumerator()
        {
            FtpWebRequest request = this.owner.GetFtpRequest(this.viewPath);
            request.Method = WebRequestMethods.Ftp.ListDirectoryDetails;

            // Ottengo la risposta con la lista
            using (FtpWebResponse response = (FtpWebResponse)request.GetResponse())
            {
                using (Stream stream = response.GetResponseStream())
                {
                    // Parserizzo il risultato
                    return this.owner.Parser.ParseListDirectoryDetails(stream, this.owner.ShowHierarchicalFiles).GetEnumerator();
                }
            }
        }

        #endregion
    }

    
}
