Imports System
Imports System.Data
Imports System.Configuration
Imports System.Web
Imports System.Web.Security
Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports System.Web.UI.WebControls.WebParts
Imports System.Web.UI.HtmlControls
Imports System.Collections
Imports System.IO

Namespace ASPItalia.Books.Web.Chapter11

    '/ <summary>
    '/ DataSource che permette l'accesso ai files e alle directory
    '/ </summary>
    Public Class FileSystemDataSource
        Inherits HierarchicalDataSourceControl
        Implements IDataSource

        Private _viewNames() As String

        Protected Overrides Function GetHierarchicalView(ByVal viewPath As String) As HierarchicalDataSourceView
            Return New FileSystemHierarchicalDataSourceView(Me, viewPath)
        End Function

        '/ <summary>
        '/ Percorso assoluto o relativo della directory base dalla quale sfogliare
        '/ </summary>
        Public Property BaseDirectory() As String
            Get
                Dim o As Object = ViewState("BaseDirectory")
                If Not o Is Nothing Then
                    Return CType(o, String)
                End If
                Return String.Empty
            End Get
            Set(ByVal Value As String)
                ViewState("BaseDirectory") = value

                Me.OnDataSourceChanged(EventArgs.Empty)
            End Set
        End Property

        '/ <summary>
        '/ Indica se mostrare in modo gerarchico anche i files. Default false
        '/ </summary>
        Public Property ShowHierarchicalFiles() As Boolean
            Get
                Dim o As Object = ViewState("ShowHierarchicalFiles")
                If Not o Is Nothing Then
                    Return CType(o, Boolean)
                Else
                    Return False
                End If
            End Get
            Set(ByVal Value As Boolean)
                ViewState("ShowHierarchicalFiles") = True

                Me.OnDataSourceChanged(EventArgs.Empty)
            End Set
        End Property

        Friend ReadOnly Property BaseDirectoryInfo() As DirectoryInfo
            Get
                Dim path As String = Me.BaseDirectory
                If Not System.IO.Path.IsPathRooted(path) Then
                    path = Me.Context.Request.MapPath(path)
                End If
                Return New DirectoryInfo(path)
            End Get
        End Property


#Region "IDataSource Members"

        Private Function GetView(ByVal viewName As String) As DataSourceView Implements IDataSource.GetView
            If viewName.Length = 0 Then
                viewName = "DefaultView"
            End If

            Return New FileSystemDataSourceView(Me, viewName)
        End Function

        Private Function GetViewNames() As ICollection Implements IDataSource.GetViewNames
            If Me._viewNames Is Nothing Then
                _viewNames = New String() {"DefaultView"}
            End If

            Return Me._viewNames
        End Function

        Custom Event DataSourceChanged As EventHandler Implements IDataSource.DataSourceChanged
            AddHandler(ByVal value As EventHandler)
                AddHandler DirectCast(Me, IHierarchicalDataSource).DataSourceChanged, value
            End AddHandler
            RemoveHandler(ByVal value As EventHandler)
                RemoveHandler DirectCast(Me, IHierarchicalDataSource).DataSourceChanged, value
            End RemoveHandler
            RaiseEvent(ByVal sender As Object, ByVal e As EventArgs)

            End RaiseEvent
        End Event

#End Region

    End Class

End Namespace
