Imports System
Imports System.Data
Imports System.Configuration
Imports System.Web
Imports System.Web.Security
Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports System.Web.UI.WebControls.WebParts
Imports System.Web.UI.HtmlControls
Imports System.Collections
Imports System.IO

Namespace ASPItalia.Books.Web.Chapter11
    '/ <summary>
    '/ Vista per il FileSystemDataSource
    '/ </summary>
    Public Class FileSystemHierarchicalDataSourceView
        Inherits HierarchicalDataSourceView

        Private owner As FileSystemDataSource
        Private viewPath As String

        Friend Sub New(ByVal owner As FileSystemDataSource, ByVal viewPath As String)
            Me.owner = owner
            Me.viewPath = viewPath
        End Sub

        Public Overrides Function [Select]() As IHierarchicalEnumerable
            ' Controllo la validit della directory
            If Me.owner.BaseDirectory.Length = 0 Then
                Throw New HttpException(String.Format("Invalid BaseDirectory for control {0}", Me.owner.ID))
            End If

            Dim path As DirectoryInfo = Me.owner.BaseDirectoryInfo
            Dim parent As DirectoryInfo = Nothing

            If Not String.IsNullOrEmpty(viewPath) Then
                Dim steps() As String = viewPath.Split(New Char() {"/"c}, StringSplitOptions.RemoveEmptyEntries)
                ' Entro ricorsivamente all'interno della directory
                Dim [step] As String
                For Each [step] In steps
                    ' Memorizzo il precedente path come padre
                    parent = path
                    ' Cerco la sottodirectory
                    Dim childDirs() As DirectoryInfo = path.GetDirectories([step], SearchOption.TopDirectoryOnly)
                    If childDirs.Length = 0 Then
                        Throw New HttpException(String.Format("Child directory {0} in {1} does not exist", [step], path.FullName))
                    End If

                    path = childDirs(0)
                Next
            End If

            ' Restituisco l'enumeratore per quella directory
            Return New FileSystemHierarchicalEnumerable(path, Nothing, viewPath, Me)
        End Function

        Friend ReadOnly Property ShowHierarchicalFiles() As Boolean
            Get
                Return Me.owner.ShowHierarchicalFiles
            End Get
        End Property
    End Class

End Namespace
