Imports System.Web
Imports System.Web.Security
Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports System.Web.UI.WebControls.WebParts
Imports System.Web.UI.HtmlControls
Imports System.Collections
Imports System.Net
Imports System.IO

Namespace ASPItalia.Books.Web.Chapter11

    Public Class FtpHierarchicalDataSourceView
        Inherits HierarchicalDataSourceView

        Private owner As FtpDataSource
        Private viewPath As String

        Friend Sub New(ByVal owner As FtpDataSource, ByVal viewPath As String)
            MyBase.New()
            Me.owner = owner
            Me.viewPath = viewPath
        End Sub

        Public Overrides Function [Select]() As IHierarchicalEnumerable
            Return New FtpHierarchicalEnumerable(Me.owner, Nothing, Me.viewPath)
        End Function
    End Class

    Public Class FtpHierarchicalEnumerable
        Implements IHierarchicalEnumerable

        Private owner As FtpDataSource
        Private parent As FtpHierarchyData
        Private viewPath As String

        Friend Sub New(ByVal owner As FtpDataSource, ByVal parent As FtpHierarchyData, ByVal viewPath As String)
            Me.owner = owner
            Me.parent = parent
            Me.viewPath = viewPath
        End Sub

#Region "IHierarchicalEnumerable Members"

        Private Function GetHierarchyData(ByVal enumeratedItem As Object) As IHierarchyData Implements IHierarchicalEnumerable.GetHierarchyData
            Dim file As FtpFileInfo = DirectCast(enumeratedItem, FtpFileInfo)
            Return New FtpHierarchyData(Me.owner, parent, file, String.Format("{0}/{1}", Me.viewPath, file.Name))
        End Function


        Public Class FtpHierarchyData
            Implements IHierarchyData

            Private file As FtpFileInfo
            Private _viewPath As String
            Private _parent As FtpHierarchyData
            Private owner As FtpDataSource

            Friend Sub New(ByVal owner As FtpDataSource, ByVal parent As FtpHierarchyData, ByVal file As FtpFileInfo, ByVal viewPath As String)
                Me.owner = owner
                Me._parent = parent
                Me.file = file
                Me._viewPath = viewPath
            End Sub

            Private Function GetChildren() As IHierarchicalEnumerable Implements IHierarchyData.GetChildren
                If Me.file.IsDirectory Then
                    Return New FtpHierarchicalEnumerable(Me.owner, Me, Me._viewPath)
                Else
                    Return Nothing
                End If
            End Function

            Private Function GetParent() As IHierarchyData Implements IHierarchyData.GetParent
                Return _parent
            End Function

            Private ReadOnly Property HasChildren() As Boolean Implements IHierarchyData.HasChildren
                Get
                    Return (Me.file.IsDirectory)
                End Get
            End Property

            Private ReadOnly Property Item() As Object Implements IHierarchyData.Item
                Get
                    Return Me.file
                End Get
            End Property

            Private ReadOnly Property Path() As String Implements IHierarchyData.Path
                Get
                    Return Me._viewPath
                End Get
            End Property


            Private ReadOnly Property Type() As String Implements IHierarchyData.Type
                Get
                    If Me.file.IsDirectory Then
                        Return "Directory"
                    Else
                        Return "File"
                    End If
                End Get
            End Property

        End Class

#End Region

#Region "IEnumerable Members"

        Private Function GetEnumerator() As IEnumerator Implements IEnumerable.GetEnumerator

            Dim request As FtpWebRequest = Me.owner.GetFtpRequest(Me.viewPath)
            request.Method = WebRequestMethods.Ftp.ListDirectoryDetails

            ' Ottengo la risposta con la lista
            Using response As FtpWebResponse = DirectCast(request.GetResponse(), FtpWebResponse)

                Using stream As Stream = response.GetResponseStream()
                    ' Parserizzo il risultato
                    Return Me.owner.Parser.ParseListDirectoryDetails(stream, Me.owner.ShowHierarchicalFiles).GetEnumerator()
                End Using
            End Using
        End Function

#End Region
    End Class

End Namespace
