Imports System
Imports System.Collections.Generic
Imports System.Data
Imports System.Configuration
Imports System.Globalization
Imports System.IO
Imports System.Text.RegularExpressions
Imports System.Web
Imports System.Web.Security
Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports System.Web.UI.WebControls.WebParts
Imports System.Web.UI.HtmlControls
Imports System.Text

namespace ASPItalia.Books.Web.Chapter11

    Public Class FtpParser

        Private ReadOnly encoding As Encoding

        Private directoryDetailsRegex As New Regex("(?i)(?<date>\d{2}-\d{2}-\d{2}\s{2}\d{2}:\d{2}((AM)|(PM)))" & _
"\s{7}" & _
"(?<dir>\<dir\>)?" & _
"\s*" & _
"(?<size>\d+)?" & _
"\s" & _
"(?<name>[^\r\n]+)", RegexOptions.Compiled Or RegexOptions.IgnorePatternWhitespace)

        Public Sub New(ByVal encoding As Encoding)

            Me.encoding = encoding
        End Sub

        Public Function ParseListDirectoryDetails(ByVal stream As Stream) As List(Of FtpFileInfo)
            Return Me.ParseListDirectoryDetails(stream, True)
        End Function

        Public Function ParseListDirectoryDetails(ByVal stream As Stream, ByVal showFiles As Boolean) As List(Of FtpFileInfo)
            Dim list As New List(Of FtpFileInfo)

            Using reader As New StreamReader(stream)

                Dim line As String
                ' Carico una riga per volta
                line = reader.ReadLine()
                While Not line Is Nothing
                    line = reader.ReadLine()
                    ' Cerco i campi con una regex
                    Dim m As Match = directoryDetailsRegex.Match(line)
                    If m.Success Then

                        Dim size As Integer = 0
                        ' Provo a parserizzare la dimensione
                        If m.Groups("size").Length > 0 Then
                            Int32.TryParse(m.Groups("size").Value, size)
                        End If

                        ' Provo a caricare la data
                        Dim lastModified As DateTime
                        DateTime.TryParse(m.Groups("date").Value, CultureInfo.CreateSpecificCulture("en-US"), DateTimeStyles.AssumeLocal, lastModified)

                        ' Creo l'oggetto che lo rappresenta
                        Dim file As FtpFileInfo = New FtpFileInfo(m.Groups("name").Value, size, (m.Groups("dir").Length > 0), lastModified)
                        If showFiles Or file.IsDirectory Then
                            list.Add(file)
                        End If
                    End If
                End While
            End Using

            Return list
        End Function

    End Class

    Public Class FtpFileInfo

        Private _name As String
        Private _size As Integer
        Private _isDirectory As Boolean
        Private _lastModified As DateTime

        Public Sub New(ByVal name As String, ByVal size As Integer, ByVal isDirectory As Boolean, ByVal lastModified As DateTime)
            _name = name
            _size = size
            _isDirectory = isDirectory
            _lastModified = lastModified
        End Sub

        Public ReadOnly Property Name() As String
            Get
                Return _name
            End Get
        End Property

        Public ReadOnly Property Size() As Integer
            Get
                Return _size
            End Get
        End Property

        Public ReadOnly Property IsDirectory() As Boolean
            Get
                Return _isDirectory
            End Get
        End Property

        Public ReadOnly Property LastModified() As DateTime
            Get
                Return _lastModified
            End Get
        End Property
    End Class

end namespace