Imports System
Imports System.Data
Imports System.Configuration
Imports System.Web
Imports System.Web.Security
Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports System.Web.UI.WebControls.WebParts
Imports System.Web.UI.HtmlControls
Imports System.ComponentModel


Namespace ASPItalia.Books.Web.Chapter11

    Public Class SizeField
        Inherits DataControlField

        Private _dataField As String
        Private fieldDescriptor As PropertyDescriptor

        Protected Overrides Function CreateField() As System.Web.UI.WebControls.DataControlField
            Return New SizeField()
        End Function

        Public Overrides Sub InitializeCell(ByVal cell As DataControlFieldCell, ByVal cellType As DataControlCellType, ByVal rowState As DataControlRowState, ByVal rowIndex As Integer)
            ' Solo se non  ne un footer ne un header
            If cellType = DataControlCellType.DataCell Then
                Dim control As Control
                ' Solo se non sono in modalit di modifica o inserimento
                If (rowState Or DataControlRowState.Edit) <> DataControlRowState.Edit Then
                    control = cell
                Else
                    ' Creo la textbox di modifica
                    Dim tb As New TextBox()
                    tb.Columns = 5
                    tb.Text = "0"
                    cell.Controls.Add(tb)
                    control = tb
                End If

                ' Intercetto l'evento DataBinding del controllo
                If Not control Is Nothing Then
                    AddHandler control.DataBinding, New EventHandler(AddressOf control_DataBinding)
                End If
            End If

            MyBase.InitializeCell(cell, cellType, rowState, rowIndex)
        End Sub

        Public Overrides Sub ExtractValuesFromCell(ByVal dictionary As System.Collections.Specialized.IOrderedDictionary, ByVal cell As DataControlFieldCell, ByVal rowState As DataControlRowState, ByVal includeReadOnly As Boolean)
            If (rowState Or DataControlRowState.Insert) = DataControlRowState.Insert Or Me.InsertVisible Then

                ' Se la  cella non ha figli allora  in modalit lettura
                ' quindi prelevo il suo testo
                Dim value As Object = Nothing
                If cell.Controls.Count = 0 Then
                    value = cell.Text
                ElseIf TypeOf cell.Controls(0) Is TextBox Then ' Altrimenti  una TextBox e leggo il suo contenuto
                    value = DirectCast(cell.Controls(0), TextBox).Text
                End If

                ' Aggiungo nel dictionary per il relativo campo il valore
                Dim key As String = Me.DataField
                If dictionary.Contains(key) Then ' Se gi lo sostituisco
                    dictionary(key) = value
                Else
                    dictionary.Add(key, value)
                End If
            End If
        End Sub

        Sub control_DataBinding(ByVal sender As Object, ByVal e As EventArgs)
            Dim control As Control = DirectCast(sender, Control)
            Dim container As Control = control.NamingContainer

            Dim value As Object = Me.GetValue(container)
            ' Se  in modalit lettura, cambio il contenuto della cella
            If TypeOf control Is TableCell Then
                Dim text As String = FormatValue(value)
                DirectCast(control, TableCell).Text = text
            ElseIf TypeOf control Is TextBox Then
                DirectCast(control, TextBox).Text = value.ToString()
            End If
        End Sub

        Function FormatValue(ByVal value As Object) As String
            If value Is Nothing Or Convert.IsDBNull(value) Then
                value = 0
            End If
            Dim size As Integer = Convert.ToInt32(value)

            Dim s As Double = 0

            ' Trova l'esponente arrotondando per difetto
            Dim e As Integer
            If size <= 0 Then
                e = 0
            Else
                e = CType(Math.Floor(Math.Log(size, 1024)), Integer)
            End If

            ' In base all'esponente determino la dimensione
            Dim type As String = Nothing
            Select Case e
                Case 1
                    type = "KB"
                Case 2
                    type = "MB"
                Case 3
                    type = "GB"
                Case 4
                    type = "TB"
                Case Else
                    type = "Bytes"
            End Select

            ' Divido per 1024 elvato per l'esemponente
            s = Math.Round(size / Math.Pow(1024, e))
            Return String.Format("{0} {1}", s, type)
        End Function

        Function GetValue(ByVal container As Control) As Object

            If Me.DesignMode Then Return 0

            Dim dataItem As Object = DataBinder.GetDataItem(container)

            ' Cerco i metadati della propriet se non  stato ancora fatto
            ' ottimizzo la ricerca via reflection salvando le informazioni
            If fieldDescriptor Is Nothing Then
                fieldDescriptor = TypeDescriptor.GetProperties(dataItem).Find(Me.DataField, True)
                If fieldDescriptor Is Nothing Then
                    Throw New HttpException(String.Format("Cannot find property {0}", Me.DataField))
                End If
            End If

            Return fieldDescriptor.GetValue(dataItem)

        End Function

        <TypeConverter("System.Web.UI.Design.DataSourceViewSchemaConverter, System.Design, Version=2.0.0.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a"), Category("Data"), DefaultValue("")> _
        Public Overridable Property DataField() As String
            Get
                If Me._dataField Is Nothing Then
                    Dim o As Object = MyBase.ViewState("DataField")
                    If Not o Is Nothing Then
                        Me._dataField = CType(o, String)
                    End If
                Else
                    Me._dataField = String.Empty
                End If
                Return Me._dataField
            End Get
            Set(ByVal Value As String)
                If Not Object.Equals(value, MyBase.ViewState("DataField")) Then
                    MyBase.ViewState("DataField") = value
                    Me._dataField = value
                    Me.OnFieldChanged()
                End If
            End Set
        End Property

    End Class

End Namespace