Imports System
Imports System.Collections.Generic
Imports System.Text
Imports System.Web
Imports System.Drawing
Imports System.Drawing.Imaging
Imports System.Net.Mime
Imports System.IO
Imports System.Web.Hosting

Namespace ASPItalia.Books.Web.Chapter14

    Public Class ImageHandlerFactory
        Implements IHttpHandlerFactory

#Region "IHttpHandlerFactory Members"

        Private Function GetHandler(ByVal context As HttpContext, ByVal requestType As String, ByVal url As String, ByVal pathTranslated As String) As IHttpHandler Implements IHttpHandlerFactory.GetHandler
            ' Guardo se  di mia competenza
            If String.Compare(VirtualPathUtility.GetDirectory(context.Request.Path), VirtualPathUtility.ToAbsolute("~/images/"), True) = 0 Then
                Return New ImageHandler()
            Else
                Return New DefaultHttpHandler()
            End If
        End Function

        Private Sub ReleaseHandler(ByVal handler As IHttpHandler) Implements IHttpHandlerFactory.ReleaseHandler
        End Sub

#End Region

    End Class

    Public Class ImageHandler
        Implements IHttpHandler

        Private _jpegEncoder As ImageCodecInfo
        Private _jpegEncoderParams As EncoderParameters

        Private Const margin As Integer = 5

#Region "IHttpHandler Members"

        Private ReadOnly Property IsReusable() As Boolean Implements IHttpHandler.IsReusable
            Get
                Return True
            End Get
        End Property

        Private Sub ProcessRequest(ByVal context As HttpContext) Implements IHttpHandler.ProcessRequest
            ' Apro il file in lettura
            Using stream As Stream = File.OpenRead(context.Request.PhysicalPath)
                ' Carico l'immagine fisica
                Using image As Image = System.Drawing.Image.FromStream(stream)
                    ' Ridimensiono l'immagine
                    Dim newImage As Image = ResizeImageIfNeed(context, image)

                    ' Salvo l'immagine sul buffer di uscita
                    context.Response.ContentType = MediaTypeNames.Image.Jpeg
                    context.Response.BufferOutput = False
                    newImage.Save(context.Response.OutputStream, JpegEncoder, JpegEncoderParams)
                    context.Response.Flush()

                End Using
            End Using
        End Sub

#End Region


        Private Function ResizeImageIfNeed(ByVal context As HttpContext, ByVal image As Image) As Image
            Dim width As Integer
            Dim height As Integer

            ' Controllo i parametri di resize
            Dim widthResize As Boolean = Int32.TryParse(context.Request.QueryString("w"), width)
            Dim heightResize As Boolean = Int32.TryParse(context.Request.QueryString("h"), height)

            ' Se non  stato richiesto nessun resize ritorno l'immagine originale
            If Not widthResize And Not heightResize Then
                Return image
            ElseIf widthResize And Not heightResize Then
                ' Se  stato richiesto un resize sulla larghezza, calcolo l'altezza proporzionalmente
                height = width * image.Height / image.Width
            ElseIf Not widthResize And heightResize Then
                ' Se  stato richiesto un resize sulla l'altezza, calcolo la larghezza proporzionalmente
                width = height * image.Width / image.Height
            End If

            ' Ritorno una nuova bitmap con le nuove dimensioni
            Return New Bitmap(image, width, height)
        End Function



        Private ReadOnly Property JpegEncoder() As ImageCodecInfo
            Get
                If _jpegEncoder Is Nothing Then
                    SyncLock Me
                        If _jpegEncoder Is Nothing Then
                            Dim codec As ImageCodecInfo
                            For Each codec In ImageCodecInfo.GetImageEncoders()
                                If String.Compare(codec.MimeType, MediaTypeNames.Image.Jpeg, True) = 0 Then
                                    _jpegEncoder = codec
                                    Exit For
                                End If
                            Next
                        End If
                    End SyncLock
                End If

                Return _jpegEncoder
            End Get
        End Property


        Private ReadOnly Property JpegEncoderParams() As EncoderParameters
            Get
                If _jpegEncoderParams Is Nothing Then
                    SyncLock Me
                        If _jpegEncoderParams Is Nothing Then
                            _jpegEncoderParams = New EncoderParameters(1)
                            _jpegEncoderParams.Param(0) = New EncoderParameter(System.Drawing.Imaging.Encoder.Quality, 100L)
                        End If
                    End SyncLock
                End If
                Return _jpegEncoderParams
            End Get
        End Property

    End Class
End Namespace
